--[[****************************************************************************
  * WoWHead Linker by Fredro                                                   *
  * wowheadlinker.lua - Converts wowhead.com item urls to in-game links        *
  
  Copyright (C) 2015 Fredrik Lundberg (Fredro)

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ****************************************************************************]]
local frame = CreateFrame("Frame", nil, UIParent);
local waitingOnItemData = {};

frame:RegisterEvent("GET_ITEM_INFO_RECEIVED");
frame:SetScript("OnEvent", function(self, event, ...)
	if event == "GET_ITEM_INFO_RECEIVED" then
		gotEvents();
	end
end);

SLASH_WOWHEADLINKER1 = "/wh";
local function handler(msg, editbox)
	local command, url = msg:match("^(%S*)%s*(.-)$");
	
	if command == "" or url == "" then
		print("|cff00ff00 WoWHeadLinker Syntax:|cffff0000 /wh |r (|cffff0000 me |r(self) | |cffff0000 s |r (say) | |cffff0000 player name |r | |cffff0000 1 |r (general) | |cffff0000 2 |r (trade) | |cffff0000 p |r (party) | |cffff0000 r |r (raid) | |cffff0000 g |r (guild) ) |cffff0000 http://www.wowhead.com/item=XXX |r or |cffff0000 http://www.wowhead.com/spell=XXX |r or |cffff0000 http://www.wowhead.com/achievement=XXX |r");
		return;
	end
	local itemInfo
	if string.find(url, "item") then
		local itemNr = string.match(url,"%d+");
		itemInfo = cacheItem(itemNr,"item", command);
	elseif string.find(url, "spell") then
		local spellNr = string.match(url,"%d+");
		itemInfo = cacheItem(spellNr, "spell", command);
	elseif string.find(url, "achievement") then
		local achiNr = string.match(url,"%d+");
		itemInfo = cacheItem(achiNr, "achievement", command);
	else
		print("|cffff0000 WoWHeadLinker: |r Bad URL! Item not supported");
		return;
	end
	if itemInfo ~= nil then
		printItem(itemInfo, command);
	end
end

function cacheItem(itemID, type, command)
	if type == "item" then
		itemLink = select(2,GetItemInfo(itemID));
	elseif type == "spell" then
		itemLink = select(1,GetSpellLink(itemID));
	elseif type == "achievement" then
		itemLink = select(1,GetAchievementLink(itemID));
	end
	if itemLink == nil then
		table.insert(waitingOnItemData, {itemID, type, command})
	else
		return itemLink;
	end
end

function gotEvents()
	if waitingOnItemData[1] == nil then
		--print
	else
		itemInfo = cacheItem(waitingOnItemData[1][1], waitingOnItemData[1][2], waitingOnItemData[1][3]);
		if itemInfo == nil then
			print("not cached");
			return;
		else
			printItem(itemInfo, waitingOnItemData[1][3]);
		end
		table.remove(waitingOnItemData, 1);
	end
end

function printItem( itemInfo, command )
	if itemInfo == nil then
		print("Item is not in your cache, just re-run the command again");
		return;
	end
	if command == "self" or command == "me" and rest ~= "" then
		DEFAULT_CHAT_FRAME:AddMessage("|cff00ff00 WoWHeadLinker: |r " .. itemInfo);
	elseif command == "say" or command == "s" and rest ~= "" then
		SendChatMessage(itemInfo,"SAY");
	elseif command == "trade" or command == "2" and rest ~= "" then
		local trID1 = select(1,GetChannelList());
		local trID2 = select(3,GetChannelList());
		if trID1 == 2 or trID2 == 2 then
			SendChatMessage(itemInfo,"CHANNEL",nil,2);
		else
			print("|cffff0000 WoWHeadLinker Error: |r Not in a Trade channel at the moment");
		end
	elseif command == "general" or command == "1" and rest ~= "" then
		local genID = select(1,GetChannelList());
		if genID == 1 then
			SendChatMessage(itemInfo,"CHANNEL",nil,1);
		else
			print("|cffff0000 WoWHeadLinker Error: |r Not in a General channel at the moment");
		end

	elseif command ~= "" and rest ~= "" and command ~="self" and command ~="say" and command ~="party" and command ~="raid" and command ~="trade" and command ~="2" and command ~="guild" and command ~="1" and command ~="g" and command ~="r" and command ~="p" and command ~="s" and command ~="me" then
		SendChatMessage(itemInfo,"WHISPER", nil, command);
	elseif command == "party" or command == "p" and rest ~= "" then
		SendChatMessage(itemInfo,"PARTY");
	elseif command == "raid" or command == "r" and rest ~= "" then
		SendChatMessage(itemInfo,"RAID");
	elseif command == "guild" or command == "g" and rest ~= "" then
		SendChatMessage(itemInfo,"GUILD");
	else
		print("|cff00ff00 WoWHeadLinker Syntax:|cffff0000 /wh |r (|cffff0000 me |r(self) | |cffff0000 s |r (say) | |cffff0000 player name |r | |cffff0000 1 |r (general) | |cffff0000 2 |r (trade) | |cffff0000 p |r (party) | |cffff0000 r |r (raid) | |cffff0000 g |r (guild) ) |cffff0000 http://www.wowhead.com/item=XXX |r or |cffff0000 http://www.wowhead.com/spell=XXX |r or |cffff0000 http://www.wowhead.com/achievement=XXX |r");
	end
end

SlashCmdList["WOWHEADLINKER"] = handler;


